package org.dromara.service;

import org.dromara.domain.bo.SenEventsObjectBo;
import org.dromara.domain.vo.SenEventsVo;
import org.dromara.domain.bo.SenEventsBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 事件信息Service接口
 *
 * @author luogang
 * @date 2024-09-19
 */
public interface ISenEventsService {

    /**
     * 查询事件信息
     *
     * @param id 主键
     * @return 事件信息
     */
    SenEventsVo queryById(Long id);

    /**
     * 分页查询事件信息列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 事件信息分页列表
     */
    TableDataInfo<SenEventsVo> queryPageList(SenEventsBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的事件信息列表
     *
     * @param bo 查询条件
     * @return 事件信息列表
     */
    List<SenEventsVo> queryList(SenEventsBo bo);

    /**
     * 新增事件信息
     *
     * @param bo 事件信息
     * @return 是否新增成功
     */
    Boolean insertByBo(SenEventsBo bo);

    /**
     * 修改事件信息
     *
     * @param bo 事件信息
     * @return 是否修改成功
     */
    Boolean updateByBo(SenEventsBo bo);

    /**
     * 校验并批量删除事件信息信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
    /**
     * 查询事件数量
     */
    HashMap<String, Object> eventsCount();
    /**
     * 重点事件统计
     */
    List<HashMap<String,Object>> importantEventsStat(String dateType);

    /**
     * 机占非TOP10
     */
   HashMap<String, Object> motorInLanesTop10(String dateType);
    /**
     * 事件占比TOP10（%）
     */
    List<Map<String,Object>> eventPercentTop10();
    /**
     * 近30日事件统计
     */
    List<Map<String, Object>> eventMonthCount();
    /**
     * 每日事件统计
     */
    List<Map<String, Object>> eventDayCount();
    /**
     * 监控点位违反统计
     */
    List<HashMap<String, Object>> pointEventsStat(String dateType);
    /**
     * 区域违法分布
     */
    List<HashMap<String, Object>> regionEventsStat(String dateType);
    /**
     * 每分钟事件统计
     */
    List<HashMap<String, Object>> minuteEventsStat(String eventType);
    /**
     * 交通违法数据来源分析
     */
    List<HashMap<String, Object>> eventsDatasourceStat(String dateType);
    /**
     * 交通违法分布式时间
     */
    List<HashMap<String, Object>> eventsHourStat(String dateType);
    /**
     * 交通违法处置分析
     */
    List<HashMap<String, Object>> eventsDealRateStat(String dateType);
    /**
     * 骑行违法分析
     */
    TableDataInfo<Map<String, Object>> cycleEventStat(SenEventsBo senEventsBo, PageQuery pageQuery);
    /**
     * 重点车辆分析
     */
    TableDataInfo<Map<String, Object>> keyVehicleEventStat(SenEventsObjectBo bo, PageQuery pageQuery);

    TableDataInfo<SenEventsVo> eventsMediaList(SenEventsBo bo, PageQuery pageQuery);
}
